package de.uni.leipzig.informatik.pacosy.datastructure.dao;

import de.uni.leipzig.informatik.pacosy.datastructure.api.IAssociation;
import de.uni.leipzig.informatik.pacosy.datastructure.api.INode;

public class Association implements IAssociation {

	Node primaryNode;
	Node secondaryNode;
	Double strength;

	public Association() {
		this.primaryNode = null;
		this.secondaryNode = null;
		this.strength = null;
	}
	
	public Association(Node primaryNode, Node secondaryNode) {
		this.primaryNode = primaryNode;
		this.secondaryNode = secondaryNode;
		this.strength = null;
	}

	@Override
	public Node getPrimaryNode() {
		return this.primaryNode;
	}

	@Override
	public void setPrimaryNode(INode primaryNode) {
		this.primaryNode = (Node) primaryNode;
	}

	@Override
	public Node getSecondaryNode() {
		return this.secondaryNode;
	}

	@Override
	public void setSecondaryNode(INode secondaryNode) {
		this.secondaryNode = (Node) secondaryNode;
	}

	@Override
	public double getStrength() {
		if (this.strength==null){
			if (this.primaryNode.isLeaf() && this.secondaryNode.isLeaf()){
				this.strength = 0d;
			}
			else
				this.strength = 0.5d;
		}
		return this.strength;
	}

	@Override
	public void setStrength(double strength) {
		this.strength = strength;
	}

	public void update() {
		if (this.primaryNode != null && this.secondaryNode != null) {
			this.primaryNode.addAssociations(this.secondaryNode, this.strength);
			this.secondaryNode.addAssociations(this.primaryNode, this.strength);
		}
	}

	public void remove() {
		if (this.primaryNode != null && this.secondaryNode != null) {
			this.primaryNode.getAssociations().remove(this.secondaryNode);
			this.secondaryNode.getAssociations().remove(this.primaryNode);
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((primaryNode == null) ? 0 : primaryNode.hashCode());
		result = prime * result
				+ ((secondaryNode == null) ? 0 : secondaryNode.hashCode());
		long temp;
		temp = Double.doubleToLongBits(strength);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Association other = (Association) obj;
		if (primaryNode == null) {
			if (other.primaryNode != null)
				return false;
		} else if (!primaryNode.equals(other.primaryNode))
			return false;
		if (secondaryNode == null) {
			if (other.secondaryNode != null)
				return false;
		} else if (!secondaryNode.equals(other.secondaryNode))
			return false;
		if (strength == null) {
			if (other.strength != null)
				return false;
		} else if (!strength.equals(other.strength))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "(" + primaryNode.id + "," + secondaryNode.id + "):" + strength;
	}

}
