package de.uni.leipzig.informatik.pacosy.datastructure.dao;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.uni.leipzig.informatik.pacosy.datastructure.api.IEdge;
import de.uni.leipzig.informatik.pacosy.datastructure.api.INode;
import de.uni.leipzig.informatik.pacosy.datastructure.api.ITree;
import de.uni.leipzig.informatik.pacosy.util.Tupel;

public class Tree implements ITree {

	String id;
	Node root;
	boolean rooted;
	Map<String, Node> nodes;
	Map<Tupel<String, String>,Edge> edges;
	
	public Tree(){
		this.root = null;
		this.rooted = true;
		this.nodes = new HashMap<String, Node>();
		this.edges = new HashMap<Tupel<String,String>, Edge>();
	}
	
	@Override
	public String getId() {
		return this.id;
	}

	@Override
	public void setId(String id) {
		this.id = id;

	}
	
	@Override
	public Node getRoot() {
		return this.root;
	}

	@Override
	public void setRoot(INode node) {
		Node n = (Node)node;
		if (n.getParent()!=null){
			recursiveSwap(n.getParent(), n);
		}
		n.setParent(null);
		this.root = n;
	}

	@Override
	public void setRoot(IEdge edge) {
		Edge e = (Edge) edge;
		if (this.root.getChildren().size()==2)
		{
			if (!e.getSource().equals(this.root)){
				//clean up old root
				Node firstChild = this.root.getChildren().get(0);
				Node secondChild = this.root.getChildren().get(1);
				this.root.getChildren().clear();
				//temporarily firstChild gets new root
				firstChild.setParent(null);
				Edge firstEdge = getEdge(this.root.getId(), firstChild.getId());
				Edge secondEdge = getEdge(this.root.getId(), secondChild.getId());
				Edge newEdge = new Edge();
				newEdge.setSource(firstChild);
				newEdge.setTarget(secondChild);
				// FIXME handle branch lengths and bootstraps
				newEdge.setLength(firstEdge.getLength()+secondEdge.getLength());
				newEdge.setBootstrap(Math.max(firstEdge.getBootstrap(), secondEdge.getBootstrap()));
				//update edges
				removeEdge(firstEdge);
				removeEdge(secondEdge);
				addEdge(newEdge);

				//update new root node
				e.getSource().getChildren().remove(e.getTarget());
				Node sourceParent = e.getSource().getParent();
				Edge newEdge1 = new Edge();
				newEdge1.setSource(this.root);
				newEdge1.setTarget(e.getSource());
				Edge newEdge2 = new Edge();
				newEdge2.setSource(this.root);
				newEdge2.setTarget(e.getTarget());
				// FIXME handle branch lengths and bootstraps
				newEdge1.setLength(e.getLength()/2d);
				newEdge1.setBootstrap(e.getBootstrap());
				newEdge2.setLength(e.getLength()/2d);
				newEdge2.setBootstrap(e.getBootstrap());
				//update edges
				addEdge(newEdge1);
				addEdge(newEdge2);
				removeEdge(e);
				
				if (sourceParent!=null){
					recursiveSwap(sourceParent, e.getSource());
				}
			}
		} else{
			//update new root node
			Node newRoot = new Node();
			e.getSource().getChildren().remove(e.getTarget());
			Node sourceParent = e.getSource().getParent();
			Edge newEdge1 = new Edge();
			newEdge1.setSource(newRoot);
			newEdge1.setTarget(e.getSource());
			Edge newEdge2 = new Edge();
			newEdge2.setSource(newRoot);
			newEdge2.setTarget(e.getTarget());
			// FIXME handle branch lengths and bootstraps
			newEdge1.setLength(e.getLength()/2d);
			newEdge1.setBootstrap(e.getBootstrap());
			newEdge2.setLength(e.getLength()/2d);
			newEdge2.setBootstrap(e.getBootstrap());
			//update edges
			addEdge(newEdge1);
			addEdge(newEdge2);
			removeEdge(e);
			this.root = newRoot;
			
			if (sourceParent!=null){
				recursiveSwap(sourceParent, e.getSource());
			}
		}
	}
	
	private void recursiveSwap(Node parent, Node child){
		if (parent.getParent() != null) {
			recursiveSwap(parent.getParent(), parent);
		}
		
		Edge edge = getEdge(parent.getId(), child.getId());
		removeEdge(edge);
		edge.swap();
		addEdge(edge);
	}
	
	public boolean isRooted() {
		return rooted;
	}

	public void setRooted(boolean rooted) {
		this.rooted = rooted;
	}

	@Override
	public Node getNode(String nodeId) {
		return this.nodes.get(nodeId);
	}

	@Override
	public Collection<Node> getNodes() {
		return this.nodes.values();
	}

	@Override
	public void setNodes(Collection<? extends INode> nodes) {
		this.nodes.clear();
		for (INode node : nodes){
			this.nodes.put(node!=null?node.getId():"", (Node)node);
		}
	}

	public void addNode(INode node) {
		this.nodes.put(node!=null?node.getId():"", (Node)node);

	}
	
	public void removeNode(INode node) {
		this.nodes.remove(node!=null?node.getId():"");

	}

	@Override
	public Edge getEdge(String sourceId, String targetId) {
		return this.edges.get(new Tupel<String, String>(sourceId, targetId));
	}

	@Override
	public Collection<Edge> getEdges() {
		return this.edges.values();
	}

	@Override
	public void setEdges(Collection<? extends IEdge> edges) {
		this.edges.clear();
		for (IEdge edge : edges){
			this.edges.put(new Tupel<String, String>(edge.getSource()!=null?edge.getSource().getId():"", edge.getTarget()!=null?edge.getTarget().getId():""), (Edge)edge);
		}
	}

	public void addEdge(IEdge edge) {
		this.edges.put(new Tupel<String, String>(edge.getSource()!=null?edge.getSource().getId():"", edge.getTarget()!=null?edge.getTarget().getId():""),(Edge)edge);

	}
	
	public void removeEdge(IEdge edge) {
		this.edges.remove(new Tupel<String, String>(edge.getSource()!=null?edge.getSource().getId():"", edge.getTarget()!=null?edge.getTarget().getId():""));

	}
	
	public void renumber(){
		this.root.renumber(0);
	}
	
	public int getSize(){
		return root.getSubtreeSize();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((edges == null) ? 0 : edges.hashCode());
		result = prime * result + ((nodes == null) ? 0 : nodes.hashCode());
		result = prime * result + ((root == null) ? 0 : root.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Tree other = (Tree) obj;
		if (edges == null) {
			if (other.edges != null)
				return false;
		} else if (!edges.equals(other.edges))
			return false;
		if (nodes == null) {
			if (other.nodes != null)
				return false;
		} else if (!nodes.equals(other.nodes))
			return false;
		if (root == null) {
			if (other.root != null)
				return false;
		} else if (!root.equals(other.root))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return id + "=" +getRoot().toString();
	}

	public String getScaledTree(Node node) {
		if (node.children.isEmpty()){
			String str = node.getId();
			return str;
		}
		else {
			String str = "(";
			for (Node child : node.children){
				str += getScaledTree(child);
				Edge edge = this.getEdge(node.getId(), child.getId());
				str += ":"+edge.getScaledLength();
				str += ",";
			}
			str = str.substring(0, str.length()-1);
			str += ")"+node.getId();
			if (node.getParent()==null){
				str += ";";
			}
			return str;
		}
	}

}
