package de.uni.leipzig.informatik.pacosy.nexus.block;

import java.util.Map;

import de.uni.leipzig.informatik.pacosy.datastructure.api.ITanglegramm;
import de.uni.leipzig.informatik.pacosy.datastructure.api.ITree;
import de.uni.leipzig.informatik.pacosy.datastructure.dao.factory.CophylogenyFactory;
import de.uni.leipzig.informatik.pacosy.nexus.block.parser.NexusCophylogenyBlockParser;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCommand;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCophylogenyAssociationsCommand;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;

public class NexusCophylogenyBlock extends NexusBlock {

	private ITanglegramm tanglegramm;

	@Override
	protected void initialize() {
		setBlockParser(NexusCophylogenyBlockParser.getSingleton());
		setDaoFactory(CophylogenyFactory.getSingleton());
	}

	@Override
	public CophylogenyFactory getDaoFactory() {
		if (super.getDaoFactory() == null) {
			return null;
		}
		return (CophylogenyFactory) super.getDaoFactory();
	}

	@Override
	public void read(NexusBlock rawBlock, Map<String, NexusBlock> blocks)
			throws NexusFormatException {
		createTanglegramm(blocks);
		
		super.read(rawBlock, blocks);
		
		for (NexusCommand command : getCommands()) {
			if (//command.getCommandName().equalsIgnoreCase("ASSOCIATIONS")&& 
					command instanceof NexusCophylogenyAssociationsCommand) {
				this.tanglegramm.setAssociations(((NexusCophylogenyAssociationsCommand)command).getAssociations());
			}
		}
		this.tanglegramm.update();
	}
	
	private void createTanglegramm(Map<String, NexusBlock> blocks) throws NexusFormatException {
		this.tanglegramm = getDaoFactory().createTanglegramm();
		NexusTreesBlock treesBlock = null;
		if (!blocks.containsKey("TREES")){
			throw new NexusFormatException("no TREES block found");
		} else{
			try {
				treesBlock = (NexusTreesBlock)blocks.get("TREES");
			} catch (Exception e) {
				throw new NexusFormatException("could not cast TREES block to class NexusTreesBlock", e);
			}
		}
		Map<String, ITree> trees = treesBlock.getTrees();
		if (trees==null || trees.isEmpty()){
			throw new NexusFormatException("no trees found in the TREES block");
		}
		this.tanglegramm.setTrees(treesBlock.getTrees());
		this.tanglegramm.setDefaultTree(treesBlock.getDefaultTree());
	}

	public ITanglegramm getTanglegramm() {
		return this.tanglegramm;
	}

}
