package de.uni.leipzig.informatik.pacosy.nexus.command.parser;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import de.uni.leipzig.informatik.pacosy.datastructure.api.IAssociation;
import de.uni.leipzig.informatik.pacosy.datastructure.api.INode;
import de.uni.leipzig.informatik.pacosy.datastructure.api.ITree;
import de.uni.leipzig.informatik.pacosy.datastructure.api.factory.ICophylogenyFactory;
import de.uni.leipzig.informatik.pacosy.nexus.Token;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusBlock;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusTreesBlock;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCommand;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCophylogenyAssociationsCommand;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;

public class NexusCophylogenyPhiCommandParser extends
		DefaultNexusCommandParser {

	protected static NexusCophylogenyPhiCommandParser singleton;
	
	public static NexusCophylogenyPhiCommandParser getSingleton(){
		if (singleton == null){
			singleton = new NexusCophylogenyPhiCommandParser();
		}
		return singleton;
	}
	
	private NexusCophylogenyPhiCommandParser(){}
	
	@Override
	public ICophylogenyFactory getDaoFactory() {
		return (ICophylogenyFactory)super.getDaoFactory();
	}
	
	@Override
	public NexusCommand deserialize(NexusCommand rawCommand, List<NexusCommand> commands, Map<String, NexusBlock> blocks) throws NexusFormatException {
		Map<String, ITree> trees = ((NexusTreesBlock)blocks.get("TREES")).getTrees();
		List<IAssociation> list = new ArrayList<IAssociation>();
		Iterator<Token> iterator = rawCommand.getTokens().iterator();
		INode species1 = null;
		INode species2 = null;
		Token token = null;
		while (iterator.hasNext()) {
			token = iterator.next();
			if (!token.isComment()) {
				if (!token.getValue().equals(",")) {
					if (species1==null) {
						try {
							species1 = getSpecies(token.getValue(), trees);
						} catch (Exception e) {
							throw new NexusFormatException(
									"unknown species identifier " + token.getValue() + " at " + token.getPosition());
						}
					} else if (species2==null) {
						try {
							species2 = getSpecies(token.getValue(), trees);
						} catch (Exception e) {
							throw new NexusFormatException(
									"unknown species identifier " + token.getValue() + " at " + token.getPosition());
						}
					} else {
						throw new NexusFormatException(
								"',' sign expected at " + token.getPosition());
					}
				} else if (species1!=null) {
					if (species2!=null) {
						IAssociation association = getDaoFactory().createAssociation();
						association.setPrimaryNode(species1);
						association.setSecondaryNode(species2);
						association.setStrength(1d);
						list.add(association);
						species1 = null;
						species2 = null;
					} else {
						throw new NexusFormatException(
								"2nd species identifier expected at " + token.getPosition());
					}
				} else {
					throw new NexusFormatException(
							"species identifier expected at " + token.getPosition());
				}
			}
		}
		if (species1!=null) {
			if (species2!=null) {
				IAssociation association = getDaoFactory().createAssociation();
				association.setPrimaryNode(species1);
				association.setSecondaryNode(species2);
				association.setStrength(1d);
				list.add(association);
			} else {
				throw new NexusFormatException(
						"2nd species identifier expected at " + token.getPosition());
			}
		}
		return new NexusCophylogenyAssociationsCommand(rawCommand, list);
	}
	
	private INode getSpecies(String speciesId, Map<String, ITree> trees) throws NexusFormatException{
		for (ITree tree : trees.values()){
			INode species = tree.getNode(speciesId);
			if (species != null){
				return species;
			}
		}
		throw new NexusFormatException(	);
	}

}
