package de.uni.leipzig.informatik.pacosy.nexus.command.parser;

import java.util.List;
import java.util.Map;

import de.uni.leipzig.informatik.pacosy.nexus.Token;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusBlock;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCommand;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusEquationStyleCommand;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;

public class NexusEquationStyleCommandParser extends DefaultNexusCommandParser {

	protected static NexusEquationStyleCommandParser singleton;
	
	public static NexusEquationStyleCommandParser getSingleton(){
		if (singleton == null){
			singleton = new NexusEquationStyleCommandParser();
		}
		return singleton;
	}
	
	private NexusEquationStyleCommandParser(){}
	
	@Override
	public NexusCommand deserialize(NexusCommand rawCommand, List<NexusCommand> commands, Map<String, NexusBlock> blocks) throws NexusFormatException {
		String leftHandSide = "";
		String rightHandSide = "";
		boolean equalSignFound = false;
		for (Token token : rawCommand.getTokens()) {
			if (!token.isComment()) {
				if (token.getValue().equals("=") && !equalSignFound) {
					equalSignFound = true;
				} else if (token.getValue().equals("=") && equalSignFound) {
					throw new NexusFormatException(
							"unexpected '=' sign at " + token.getPosition());
				} else if (!equalSignFound) {
					leftHandSide += token.getValue();
				} else {
					rightHandSide += token.getValue();
				}
			}
		}
		return new NexusEquationStyleCommand(rawCommand, leftHandSide,
				rightHandSide);
	}

}
