package de.uni.leipzig.informatik.pacosy.nexus.parser;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.uni.leipzig.informatik.pacosy.nexus.Token;
import de.uni.leipzig.informatik.pacosy.nexus.block.NexusBlock;
import de.uni.leipzig.informatik.pacosy.nexus.command.NexusCommand;
import de.uni.leipzig.informatik.pacosy.nexus.exception.NexusFormatException;

public class NexusStructureParser {

	public NexusStructureParser() {
		super();
	}

	public List<NexusBlock> deserialize(String data)
			throws NexusFormatException {
		List<Token> tokens = new ArrayList<Token>();
		String singlePunctuationCharacter[] = { "\\(", "\\)", "\\{", "\\}",
				"/", "\\\\", ",", ";", ":", "=", "\\*", "\\+", "<", ">" };
		String singlePunctuationCharacter2[] = { "\\(", "\\)", "\\[", "\\]",
				"\\{", "\\}", "/", "\\\\", ",", ";", ":", "=", "\\*", "'",
				"\"", "\\+", "<", ">" };

		String regex = "";
		for (String spc : singlePunctuationCharacter) {
			regex += spc + "|";
		}
		regex += "\\\".*?\\\"|\\\'.*?\\\'|\\[.*?\\]";
		regex += "|[^\\s";
		for (String spc : singlePunctuationCharacter2) {
			regex += "|" + spc;
		}
		regex += "]+";
		Pattern pattern = Pattern.compile(regex, Pattern.DOTALL);
		String lines[] = data.split("\n");
		Matcher matcher = pattern.matcher(data);
		while (matcher.find()) {
			int pos = matcher.start();
			int line = getLine(pos, lines);
			int column = getColumn(pos, lines);
			Token token = new Token(matcher.group(), line, column);
			tokens.add(token);
		}

		if (tokens.size() == 0
				|| !tokens.get(0).getValue().equalsIgnoreCase("#nexus")) {
			throw new NexusFormatException("#NEXUS statement missing");
		}

		return deserializeBlocks(tokens);
	}

	private int getLine(int pos, String[] lines) {
		int counter = 0;
		for (int line = 0; line<lines.length; line++){
			if (counter>pos){
				return line;
			}
			counter+=lines[line].length()+1;
		}
		return lines.length;
	}

	private int getColumn(int pos, String[] lines) {
		int counter = 0;
		int column = 0;
		for (int line = 0; line<lines.length; line++){
			if (counter>pos){
				return column +1;
			}
			column = pos-counter;
			counter+=lines[line].length()+1;
		}
		return lines[lines.length-1].length();
	}

	private List<NexusBlock> deserializeBlocks(List<Token> tokens)
			throws NexusFormatException {
		List<NexusBlock> blocks = new ArrayList<NexusBlock>();
		List<Token> blockTokens = new ArrayList<Token>();
		NexusBlock block = new NexusBlock();
		boolean processingBlock = false;
		Iterator<Token> iterator = tokens.iterator();
		while (iterator.hasNext()) {
			Token token = iterator.next();
			if (!processingBlock) {
				if (token.getValue().equalsIgnoreCase("BEGIN")) {
					blockTokens = new ArrayList<Token>();
					block = new NexusBlock();
					processingBlock = true;
					// Block identifier token
					token = iterator.next();
					if (!token.getValue().matches("[a-zA-Z]+")) {
						throw new NexusFormatException(
								"Block identifier expected at "
										+ token.getPosition());
					}
					block.setBlockType(token.getValue());
					// optional block name token
					token = iterator.next();
					if (!token.getValue().equals(";")) {
						block.setBlockName(token.getValue());
						token = iterator.next();
					}
					// ; token
					if (!token.getValue().equals(";")) {
						throw new NexusFormatException("Semicolon expected at "
								+ token.getPosition());
					}
				}
			} else if (processingBlock) {
				if (!token.isComment()) {
					if (token.getValue().equalsIgnoreCase("END")
							|| token.getValue().equalsIgnoreCase("ENDBLOCK")) {
						// ; token
						token = iterator.next();
						if (!token.getValue().equals(";")) {
							throw new NexusFormatException(
									"Semicolon expected at "
											+ token.getPosition());
						}
						block.setTokens(blockTokens);
						blocks.add(block);
						processingBlock = false;
					} else if (!token.getValue().matches("[a-zA-Z]+")) {
						throw new NexusFormatException(
								"Command identifier expected at "
										+ token.getPosition());
					} else {
						blockTokens.add(token);
						NexusCommand command = new NexusCommand();
						command.setCommandName(token.getValue());
						List<Token> commandTokens = new ArrayList<Token>();

						token = iterator.next();
						blockTokens.add(token);
						while (!token.getValue().equals(";")) {
							commandTokens.add(token);
							token = iterator.next();
							blockTokens.add(token);
						}

						command.setTokenss(commandTokens);
						block.addCommand(command);
					}
				}
			}
		}
		return blocks;
	}

}
